function vectorization_example2()
% Looping can be especially slow when it involves repeated file access

% make a random connectivity matrix
N=1000;
data=rand(N);
connectmat=1-squareform(pdist(data,'correlation'));
figure(10); imagesc(connectmat); axis image; colorbar

% define output file
fname=fullfile(fileparts(mfilename('fullpath')),'temp1.txt');
fprintf('\nWriting %g node connectivity matrix to Pajek format...',N);

%% write it the slow way, one cell at a time
tic
fid = fopen(fname, 'w');

%%% VERTICES
fprintf(fid, '*vertices %6i \r', N);
for i = 1:N
    fprintf(fid, '%6i "%6i" \r', [i i]);
end

%%% EDGES
fprintf(fid, '*edges \r');
for i = 1:N
    for j = 1:N
        if connectmat(i,j) ~= 0
            fprintf(fid, '%6i %6i %6f \r', [i j connectmat(i,j)]);
        end
    end
end

fclose(fid)
fprintf('\nCell-wise method took %.2f seconds)',toc)

%% write it a faster way
tic
pairs=[kron(1:N,ones(1,N)); kron(ones(1,N),1:N)]; % list all coordinates
pairs=pairs(:,diff(pairs)>0); % ignore repeated coordinates (because matrix is symmetrical);
% may want to ignore other pairs too -e.g. those with sub-threshold connection strength
weights=connectmat(sub2ind(size(connectmat),pairs(1,:),pairs(2,:))); % list all connection weights

fid=fopen(fname,'w');
%%% VERTICES
fprintf(fid,'*vertices %i', N);
fprintf(fid,'\n%i "%04i" ',repmat((1:N),2,1));

%%% EDGES
fprintf(fid,'\n*edges ');
fprintf(fid,'\n%i %i %.7f',[pairs;weights]);

fclose(fid);
fprintf('\nVectorized method took %.2f seconds',toc)
    
return

